#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: backup_purge.pl, v2.0.0-2 Tue 29 Nov 2011 11:12:49 PM CET mstauber Exp $
# Copyright 2006-2011 Stauber Multimedia Design. All rights reserved.

# This handler is triggered whenever a user initiates a delete of a VPS backup
# through the GUI.

$whatami = "handler";

use CCE;
my $cce = new CCE('Domain' => 'base-vserver');
use POSIX qw(isalpha);
use Socket;
use Sys::Hostname;
use File::Spec;
use List::MoreUtils qw(uniq);
use Net::SFTP::Foreign;
use File::Basename;
use POSIX qw(strftime);
use DateTime;

# Helper to redirect SSH errors to /dev/null:
open $ssherr, '>', '/dev/null'; # or die "Unable to open /dev/null";

use File::Remote qw(:replace);       # special :replace tag
setrsh("/usr/bin/ssh");
setrcp("/usr/bin/scp");

# Debugging switch:
$DEBUG = "0";
if ($DEBUG) {
    use Sys::Syslog qw( :DEFAULT setlogsock);
}

if ($whatami eq "handler") {
    $cce->connectfd();
}
else {
    $cce->connectuds();
}

# If this is a secondary in a cluster, we can save us the bother:
if (!-d "/vz/private") {
	$cce->bye('SUCCESS');
	exit 0;
}

$event = $cce->event_object();
$event_new = $cce->event_new();
$event_old = $cce->event_old();

&debug_msg("INFO: Passed cluster check \n");
&debug_msg("INFO: event: $event->{'OID'} \n");
&debug_msg("INFO: event_new: $event_new->{'OID'} \n");
&debug_msg("INFO: event_old: $event_old->{'OID'} \n");

# Get settings out of CODB to determine the target of the remote backup (if any):
@oids = $cce->find('VserverMasterNode');
($ok, $solbackupValues) = $cce->get($oids[0]);
if ($ok) {
    # Define values:
    $vzdump_enabled		=$solbackupValues->{'vzdump_enabled'};
    $rsync_enabled		=$solbackupValues->{'rsync_enabled'};
    $MN_RemoteBackupEnabled	=$solbackupValues->{'MN_RemoteBackupEnabled'};
    $MN_LocalBackupEnabled	=$solbackupValues->{'MN_LocalBackupEnabled'};
    $MN_BackupLocalLocation	=$solbackupValues->{'MN_BackupLocalLocation'};
    $targethost                 = $solbackupValues->{'MN_BackupRemoteTargetHost'};
    $targetip                   = $solbackupValues->{'MN_BackupRemoteTargetIP'};
    $path                       = $solbackupValues->{'MN_BackupRemoteLocation'};
    $user                       = $solbackupValues->{'MN_BackupRemoteTargetUser'};
    $pass                       = $solbackupValues->{'MN_BackupRemoteTargetUserPass'};
    $port                       = $solbackupValues->{'MN_BackupRemoteTargetPort'};
    $limit                      = $solbackupValues->{'RsyncSpeedLimit'};
    $my_maxfiles                = $solbackupValues->{'vzdump_maxfiles'};

    # Assemble paths:
    $local_location_vzdump = $solbackupValues->{'MN_BackupLocalLocation'} . "/vzbackup";
    $local_location_vzdump =~ s|//|/|g;
    $local_location_vzdump =~ s|/$||g;

    $remote_location_vzdump = $solbackupValues->{'MN_BackupRemoteLocation'};
    $remote_location_vzdump =~ s|//|/|g;
    $remote_location_vzdump =~ s|/$||g;

    $local_location_rsync = $solbackupValues->{'MN_BackupLocalLocation'} . "/rsync";
    $local_location_rsync =~ s|//|/|g;
    $local_location_rsync =~ s|/$||g;

    $remote_location_rsync = $solbackupValues->{'MN_BackupRemoteLocation'};
    $remote_location_rsync =~ s|//|/|g; 
    $remote_location_rsync =~ s|/$||g; 
}

# Get the data of the Backup Object we want to delete:
($ok, $Backup) = $cce->get($event->{'OID'});
if ($ok) {
    if (($Backup->{'location'} eq "local") && ($Backup->{'origin'} eq "vzdump") && (-f $Backup->{'storage'})) {
	&debug_msg("INFO: OID $event->{'OID'} containing a local vzdump backup at $Backup->{'storage'} is going to be deleted. \n");
	system("/bin/rm -f $Backup->{'storage'}");
        # Remove CODB Object:
        ($ok) = $cce->destroy($event->{'OID'});

    }
    if (($Backup->{'location'} eq "local") && ($Backup->{'origin'} eq "rsync") && (-d $Backup->{'storage'})) {
	&debug_msg("INFO: OID $event->{'OID'} containing a local rsync backup at $Backup->{'storage'} is going to be deleted. \n");
	system("/bin/rm -Rf $Backup->{'storage'}");
        # Remove CODB Object:
        ($ok) = $cce->destroy($event->{'OID'});
    }
    if (($Backup->{'location'} eq "remote") && ($Backup->{'origin'} eq "vzdump")) {
	&debug_msg("INFO: OID $event->{'OID'} containing a remote vzdump backup at $Backup->{'storage'} is going to be deleted. \n");

	if ($user && $pass && $port && ($targethost||$targetip) && $remote_location_vzdump) {
	    if ($targethost) {
    		$target = $user . '@' . $targethost;
	    }
	    else {   
    		$target = $user . '@' . $targetip;
	    }

	    # Connect to remote storage:
	    no warnings 'Net::SFTP::Foreign';
	    $sftp = Net::SFTP::Foreign->new($target, password => $pass, port => $port, autodie => 0, stderr_fh => $ssherr);
	    $sftp->die_on_error("Unable to connect to remote host");

	    &debug_msg("INFO: Connected to $target at port $port \n");

            # Does the old file exist?
            $fh1 = $sftp->stat("$Backup->{'storage'}");
            if ($fh1) {
        	&debug_msg("INFO: File $Backup->{'storage'} exists on remote storage. \n");
                # Remove file:
                $fh2 = $sftp->remove("$Backup->{'storage'}");
                if ($fh2) {
    		    &debug_msg("INFO: Deleted $Backup->{'storage'} as requested. \n");
                    # Remove CODB Object:
                    ($ok) = $cce->destroy($event->{'OID'});
                }
            }
	}
    }
    if (($Backup->{'location'} eq "remote") && ($Backup->{'origin'} eq "rsync")) {
	&debug_msg("INFO: OID $event->{'OID'} containing a remote rsync backup at $Backup->{'storage'} is going to be deleted. \n");

	if ($user && $pass && $port && ($targethost||$targetip) && $remote_location_vzdump) {
	    if ($targethost) {
    		$target = $user . '@' . $targethost;
		$target_short = $targethost;
	    }
	    else {   
    		$target = $user . '@' . $targetip;
		$target_short = $targetip;
	    }

	    # Connect to remote storage:
	    no warnings 'Net::SFTP::Foreign';
	    $sftp = Net::SFTP::Foreign->new($target, password => $pass, port => $port, autodie => 0, stderr_fh => $ssherr);
	    $sftp->die_on_error("Unable to connect to remote host");

	    &debug_msg("INFO: Connected to $target at port $port \n");

            # Does the remote backup directory exist?
            $fh1 = $sftp->stat("$Backup->{'storage'}");
            if ($fh1) {
        	&debug_msg("INFO: File $Backup->{'storage'} exists on remote storage. \n");
                # Remove directory:
                $tgt = $Backup->{'storage'};
		my $secure = new File::Remote (rsh => "/usr/bin/ssh", rcp => "/usr/bin/scp");
		&debug_msg("Info: Using File::Remote to delete $Backup->{'storage'} directory! \n");
		$fh2 = $secure->rmdir("$target_short:/$tgt");
                if ($fh2) {
    		    &debug_msg("INFO: Deleted $Backup->{'storage'} directory as requested. ($fh2) \n");
                    # Remove CODB Object:
                    ($ok) = $cce->destroy($event->{'OID'});
                }
                else {
            	    &debug_msg("Warning: Failed to delete $Backup->{'storage'} directory! ($fh2) \n");
                }
            }
	}
    }
}

$cce->bye('SUCCESS');
exit 0;

# Debugger:
sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        $user = $ENV{'USER'};
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

