#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: control.pl,v 2.0.0-1Tue 22 Nov 2011 03:09:07 AM CET mstauber Exp $
# Copyright 2006-2011 Stauber Multimedia Design. All rights reserved.
#
# Handler which stops, starts and reboots KVM's.
#

use CCE;
use POSIX qw(isalpha);
use Sys::Virt;
use Sys::Virt::Domain;
use XML::Simple;

my $cce = new CCE('Domain' => 'base-vserver');

# Debugging switch:
$DEBUG = "0";
if ($DEBUG) {
    use Sys::Syslog qw( :DEFAULT setlogsock);
}

$cce->connectfd();

($sys_oidmasternode) = $cce->find('VserverMasterNode');
($ok, $VserverMasterNode) = $cce->get($sys_oidmasternode);
$vpsid = $VserverMasterNode->{gui_list_ip};

($sys_oidvpsstatus) = $cce->find('KVM', {'vpsname' => $vpsid});
($ok, $vps) = $cce->get($sys_oidvpsstatus);
$vpsstatus = $vps->{status};

$vserver = $cce->event_object();
$vserver_new = $cce->event_new();
$vserver_old = $cce->event_old();

&debug_msg("INFO: controlAction $vserver_new->{controlAction} for KVM $vpsid \n");

$MN_BackupRemoteTargetHost = $sys->{'MN_BackupRemoteTargetHost'};
$MN_BackupRemoteLocation = $sys->{'MN_BackupRemoteLocation'};

if (($vserver_new->{timestamp}) || ($vserver->{timestamp})) {

	if (-f "/etc/libvirt/qemu/$vpsid.xml") {
	    $hv = Sys::Virt->new(address => "", readonly => 0);
	    $dom = $hv->get_domain_by_name($vpsid);
	}

	if (($vserver_new->{controlAction} eq "start") && ($vpsstatus ne "running")) {
	    system("/usr/bin/virsh start $vpsid");
	    &debug_msg("INFO: /usr/bin/virsh start $vpsid \n");
	}
	elsif (($vserver_new->{controlAction} eq "stop") && ($vpsstatus eq "running")) {
	    system("/usr/bin/virsh shutdown $vpsid");
	    &debug_msg("INFO: /usr/bin/virsh shutdown $vpsid \n");
	}
	elsif ($vserver_new->{controlAction} eq "reboot") {
	    # Dooooh. This needs work. virsh doesn't support 'reboot'. Using 'shutdown' and then 'start' only works
	    # if we wait for the shutdown to complete. Using 'sleep' doesn't suffice there. We actually need to poll
	    # the bloody Domain state. We'll save that for later ...
	    &debug_msg("INFO: /usr/bin/virsh destroy $vpsid \n");
	    system("/usr/bin/virsh destroy $vpsid");
	    system("sleep 15");
	    &debug_msg("INFO: /usr/bin/virsh start $vpsid \n");
	    system("/usr/bin/virsh start $vpsid");
	}
	# Delete a VPS:
	elsif ($vserver_new->{controlAction} eq "deleteconfirm") {

	    # Find all backup related entries for the VPS that's supposed to get deleted and set them to disabled:
	    @backup_stuff = $cce->findx('Vserver_Backup', { 'vpsid' => $vpsid });
	    for $backup_data (@backup_stuff) {
		($ok) = $cce->set($backup_data, '',{
    		    'VserverBackupLocalEnabled' => "0",
    		    'VserverBackupRemoteEnabled' => "0"
		});
		&debug_msg("INFO: Disabling local and remote backup for $vpsid \n");
    	    }

	    # Destroy the CODB entry for this VPS:
	    &debug_msg("INFO: Destroying CODB object 'KVM' (ID: $entry) for KVM $vpsid \n");
    	    ($ok) = $cce->destroy($sys_oidvpsstatus);

	    # Do the dirty deeds: Stop VPS and destroy VPS:
	    system("/usr/bin/virsh destroy $vpsid");
	    &debug_msg("INFO: /usr/bin/virsh destroy $vpsid \n");
	    system("sleep 5");
	    system("/usr/bin/virsh undefine $vpsid");
	    &debug_msg("INFO: /usr/bin/virsh undefine $vpsid \n");

	    # Remove Disk of KVM:
	    if (-f "/vz/kvm/images/$vpsid.img") {
		&debug_msg("INFO: Deleting /vz/kvm/images/$vpsid.img \n");
		system("/bin/rm -f /vz/kvm/images/$vpsid.img");
	    }
	    # Remove KVM XML file:
	    if (-f "/etc/libvirt/qemu/$vpsid.xml") {
		&debug_msg("INFO: Deleting /etc/libvirt/qemu/$vpsid.xml \n");
		system("/bin/rm -f /etc/libvirt/qemu/$vpsid.xml");
	    }

	    # Find the VPS entry to destroy:
	    @addresses = $cce->find('KVM', { 'vpsname' => $vpsid });
	    $total = scalar(@addresses);
	    for $entry (@addresses) {
		&debug_msg("INFO: Destroying CODB object 'KVM' (ID: $entry) for KVM $vpsid \n");
    	        ($ok) = $cce->destroy($entry);
	    }
	}

	# Restore a VPS:
	# Handle case "Local Restore":
	elsif ($vserver_new->{controlAction} eq "localrestoreconfirm") {
	    &debug_msg("INFO: Confirmation received for Local restore of KVM $vpsid: $vserver_new->{controlAction} \n");
	}
	# Handle case "Remote Restore":
	elsif ($vserver_new->{controlAction} eq "remoterestoreconfirm") {
	    &debug_msg("INFO: Confirmation received for Remote restore of KVM $vpsid: $vserver_new->{controlAction} \n");
	}

	# MODIFY a VPS:
	elsif ($vserver_new->{controlAction} eq "modify") {
	    &debug_msg("INFO: Modifying settings for KVM $vpsid \n");

	    $hv = Sys::Virt->new(address => "");
	    $dom = $hv->get_domain_by_name($vpsid);

	    # create object
	    $xml = new XML::Simple(ForceArray => 1, KeepRoot => 1, NoSort => 0);

	    # read XML file
	    $config = "/etc/libvirt/qemu/$vpsid.xml";
	    $config_out = "/tmp/$vpsid.xml";
	    $data = $xml->XMLin($config);

	    &debug_msg("INFO: Reading $config \n");

	    #&debug_msg("INFO XXX: new: $vserver_new->{vcpus} old: $vserver_old->{vcpus}  is: $vserver->{vcpus} \n");

	    $mod = "0";
	    if ($vserver_new->{vcpus} ne "") {
		$mod = "1";
		&debug_msg("INFO: KVM $vpsid: Setting VCPUs to: $vserver->{vcpus}\n");
		$data->{domain}->[0]->{'vcpu'}[0] = $vserver_new->{vcpus};
	    }

	    if ($vserver_new->{memory} ne "") {
		$mod = "1";
		# Adjust memory:
		$memory = $vserver_new->{memory};
		&debug_msg("INFO: KVM $vpsid: Setting MEMORY to: $memory \n");
		$data->{domain}->[0]->{'currentMemory'}[0] = $memory;
		$data->{domain}->[0]->{'memory'}[0] = $memory;
	    }

	    if ($vserver_new->{vnc_pass} ne "") {
		$mod = "1";
		&debug_msg("INFO: KVM $vpsid: Setting VNC pass to: $vserver->{vnc_pass}\n");
		$listen = $data->{domain}->[0]->{'devices'}->[0]->{'graphics'}->[0]->{'listen'};
		$keymap = $data->{domain}->[0]->{'devices'}->[0]->{'graphics'}->[0]->{'keymap'};
		$type = $data->{domain}->[0]->{'devices'}->[0]->{'graphics'}->[0]->{'type'};
		$autoport = $data->{domain}->[0]->{'devices'}->[0]->{'graphics'}->[0]->{'autoport'};
		$port = $data->{domain}->[0]->{'devices'}->[0]->{'graphics'}->[0]->{'port'};
		$data->{domain}->[0]->{'devices'}->[0]->{'graphics'} = {'listen' => $listen, 'keymap' => $keymap, 'type' => $type, 'autoport' => $autoport, 'port' => $port, 'passwd' => $vserver_new->{vnc_pass} };
	    }

	    if ($mod == "1") {
    		&debug_msg("INFO: Writing KVM $vpsid config to $config \n");

    		# Out with the modified XML file:
    		$xml->XMLout($data,
    		    KeepRoot   => 1,
    		    OutputFile => $config_out,
    		    RootName => 'domain',
    		    XMLDecl    => "",
    		);

		&debug_msg("INFO: Stopping KVM $vpsid to apply changed config $config_out \n");
    		system("/usr/bin/virsh destroy $vpsid");

    		&debug_msg("INFO: Defining KVM $vpsid with changed config $config_out \n");
    		system("/usr/bin/virsh define $config_out");

		if ($vserver->{autostart} == "1") {
		    &debug_msg("INFO: Starting KVM $vpsid again after applying config $config_out \n");
		    system("/usr/bin/virsh start $vpsid");
		    system("/usr/bin/virsh autostart $vpsid --enable");
		}
		else {
		    &debug_msg("INFO: Leaving KVM $vpsid stopped as autostart is set to No. \n");
		    system("/usr/bin/virsh autostart $vpsid --disable");
		}
		system("/bin/rm -f $config_out");
	    }
	#### End: Modify KVM
	}
	else {
    	    &debug_msg("INFO: No controlAction caught for KVM $vpsid \n");
	}
}

&cleanup;

sub cleanup {
    # Control action also needs to be reset to 'none' once we're done - but not on delete!
    if (($vserver_new->{controlAction} ne "deleteconfirm") && ($vpsid)) {
	#($sys_oid) = $cce->find('KVM', { 'vpsname' => $vpsid });
	#($ok, $sysvs) = $cce->get($sys_oid);
	&debug_msg("INFO: SYS_OID $sys_oidvpsstatus for $vpsid \n");
	($ok) = $cce->set($sys_oidvpsstatus, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
	    'vpsname' => $vpsid,
	    'controlAction' => 'none'
	});
    }
}

# Debugger:
sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        $user = $ENV{'USER'};
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

# We need to say 'SUCCESS' or CCE will bitch:
$cce->bye('SUCCESS');
exit(0);

