#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: generate_vserver_list.pl, v2.0.0-1 Wed 19 Oct 2011 05:57:26 PM CEST mstauber Exp $
# Copyright 2006-2011 Stauber Multimedia Design. All rights reserved.

# This constructor / handler uses 'virsh' and 'vzlist' to poll just very basic information
# off the master node to get a general idea of which VPS's are running.
#
# This information needs to be supplemented later on with extra data.
#

$whatami = "handler";

use CCE;
my $cce = new CCE;

if ($whatami eq "handler") {
    $cce->connectfd();
}
else {
    $cce->connectuds();
}

# Debugging switch:
$DEBUG = "0";

# Location of the vzlist-dump:
$vzlistdump = '/tmp/vserver-list';

# Location of vserver config files (directory - with trailing slash):
$vzcfgdir = '/etc/sysconfig/vz-scripts/';

# We are starting sane:
$bad_info_detected = '0';

# Parse existing VPS and shove their data into CCE:
&main;

# Hunt for Vserver ghost objects in CODB:
&ghostbusters;

# If we detect bad info, we will try again. Up to three times.
# Then we give up as something is barfed up badly.
if (($bad_info_detected < '3') && ($bad_info_detected != '0')) {
    &main;
}

$cce->bye('SUCCESS');
exit(0);

##
# Subs:
##

sub main {

# Find out which 'Vserver' objects we already have in CODB:

    # Find all previous Vserver COD entries and store them in an array:
    @oldVserverOIDs = $cce->findx('Vserver');
    for $entry (@oldVserverOIDs) {
	($ok, $old_vserver) = $cce->get($entry);
	push (@oldVservers, $old_vserver->{'vpsid'});
    }

    # Array of VPSIDs:
    @matrix_vpsids = ();

    # Generate full vzlist-dump:
    system("vzlist -a -o vpsid,ip,hostname,status,numproc,kmemsize,kmemsize.m,kmemsize.b,kmemsize.l,kmemsize.f,lockedpages,lockedpages.m,lockedpages.b,lockedpages.l,lockedpages.f,privvmpages,privvmpages.m,privvmpages.b,privvmpages.l,privvmpages.f,shmpages,shmpages.m,shmpages.b,shmpages.l,shmpages.f,numproc,numproc.m,numproc.b,numproc.l,numproc.f,physpages,physpages.m,physpages.b,physpages.l,physpages.f,vmguarpages,vmguarpages.m,vmguarpages.b,vmguarpages.l,vmguarpages.f,oomguarpages,oomguarpages.m,oomguarpages.b,oomguarpages.l,oomguarpages.f,numtcpsock,numtcpsock.m,numtcpsock.b,numtcpsock.l,numtcpsock.f,numflock,numflock.m,numflock.b,numflock.l,numflock.f,numpty,numpty.m,numpty.b,numpty.l,numpty.f,numsiginfo,numsiginfo.m,numsiginfo.b,numsiginfo.l,numsiginfo.f,tcpsndbuf,tcpsndbuf.m,tcpsndbuf.b,tcpsndbuf.l,tcpsndbuf.f,tcprcvbuf,tcprcvbuf.m,tcprcvbuf.b,tcprcvbuf.l,tcprcvbuf.f,othersockbuf,othersockbuf.m,othersockbuf.b,othersockbuf.l,othersockbuf.f,dgramrcvbuf,dgramrcvbuf.m,dgramrcvbuf.b,dgramrcvbuf.l,dgramrcvbuf.f,numothersock,numothersock.m,numothersock.b,numothersock.l,numothersock.f,dcachesize,dcachesize.m,dcachesize.b,dcachesize.l,dcachesize.f,numfile,numfile.m,numfile.b,numfile.l,numfile.f,numiptent,numiptent.m,numiptent.b,numiptent.l,numiptent.f,diskspace,diskspace.s,diskspace.h,diskinodes,diskinodes.s,diskinodes.h,laverage,cpulimit,cpuunits,swappages,swappages.m,swappages.b,swappages.l,swappages.f,hostname | sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzlistdump");

    # Pull the full vzlist-dump into a Matrix:
    open (F, $vzlistdump) || die "Could not open $vzlistdump: $!";
    $a = 0;                             	# Matrix line number, whereas 0 is the first (useable) line of the vzlist dump
    while ($line = <F>) {

        chomp($line);
        next if $line =~ /^\s*$/;               # skip blank lines
        next if $line =~ /^VEID/;               # skip lines starting with VEID
        next if $line =~ /^CTID/;               # skip lines starting with VEID
        $matrix_name = 'VPSID';
        my (@row) = split (/\s+/, $line);
        push (@{$matrix_name}, \@row);

    	# Build an array that contains the VPSIDs:
        if ($VPSID[$a][0]) {                    # Plausibility check. We will not store anything into CCE if 'vpsid' is not set.
            $currentvpsid = $VPSID[$a][0];
	    push (@matrix_vpsids, $VPSID[$a][0]);
        }

	# Fix for ultra-long hostnames:
        $VPSID[$a][2] = $VPSID[$a][119];

	# Check if VPS private area exists. Because it may not exist anymore, as the VPS could have been migrated off or manually have been deleted:
	$vps_home_area = "/vz/private/" . $VPSID[$a][0];
	if (-d $vps_home_area ) {
	    if ($DEBUG == "1") { print "Directory $vps_home_area exists! \n"; }
	    $VPSID[$a][132] = "1";
	}
	else {
	    if ($DEBUG == "1") { print "Directory $vps_home_area is missing! \n"; }
	    $VPSID[$a][132] = "0";
	}

	# Add VPSID to @newVservers for comparision:
	push (@newVservers, $VPSID[$a][0]);

        $a++;
    }
    close(F);
    system("rm $vzlistdump");

    # At this point we have all data from VZLIST in a matrix. But there is more data to be had in the config files. We need to get that as well:

    # How many entries do we have in the Matrix?
    $matrix_entries = $#VPSID;

    # How many VPSIDs are in the Array?
    $num_matrix_vpsids = $#matrix_vpsids;

    # Set start of Matrix line:
    $matrix_line = 0;

    foreach $currentvpsid (@matrix_vpsids) {
	&readconfig;                        	# Read missing info from vz config files
        if ($whatami ne "handler") {
        	# We only run an update of the IP address allocation if we're a constructor:
               	&ip_alloc;
        }
	&feedthemonster;            		# Feed info into CCE
	$matrix_line++;
    } 

    ##
    # Get info about the current CPU utilization and the power of the node:
    ##

    # Location of the vzcpu-dump:
    $vzcpudump = '/tmp/vserver-cpuinfo';

    # Generate vzcpu-dump:
    system("vzcpucheck | sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzcpudump");

    if (-e $vzcpudump) {
        $b = 0;
        open(CPUDUMP,"$vzcpudump") || die("Could not find $vzcpudump. $!");
        while ($line = <CPUDUMP>) {
            chomp($line);
            next if $line =~ /^\s*$/; 		# skip blank lines
            if (($line =~ /^Current CPU utilization: (.*)/) && ($b == 0)) {
                $CURRENT_CPU = $1;
            }
            if (($line =~ /^Power of the node: (.*)/) && ($b == 1)) {
                $NODE_CPU = $1;
            }
            $b++;
        }
        close(CPUDUMP);
        system("rm $vzcpudump");
        &cce_update_cpu; 			# Write the info into CCE
    }
}

# There are a few options in config files that vzlist doesn't provide.
# So we need to read them from the respective config files in
# /etc/sysconfig/vz-scripts/*.conf

sub readconfig {
    # Does the config file exist?
    $cfile = "$vzcfgdir$currentvpsid.conf";

    if (-e $cfile) {
    	open(CFILE,"$cfile");
    	while ($line = <CFILE>) {
    	    chomp($line);
    	    next if $line =~ /^\s*$/; 		# skip blank lines

    	    if ($line =~ /^ONBOOT=\"([A-Za-z]\w*)\"/) {
    		$onboot = $1;
		if (($onboot eq 'yes') || ($onboot eq 'Yes')) {
    		    $VPSID[$matrix_line][120] = 'Yes';
		}
		else {
    		    $VPSID[$matrix_line][120] = 'No';
		}
	    }
    	    if ($line =~ /^CPUUNITS=\"(.*)\"/) {
    		$VPSID[$matrix_line][113] = $1;
	    }
	    if ($line =~ /^VE_ROOT=\"(.*)\"/) {
		$VPSID[$matrix_line][121] = $1;
	    }
    	    if ($line =~ /^VE_PRIVATE=\"(.*)\"/) {
        	$VPSID[$matrix_line][122] = $1;
    	    }
    	    if ($line =~ /^OSTEMPLATE=\"(.*)\"/) {
        	$VPSID[$matrix_line][123] = $1;
	    }
    	    if ($line =~ /^ORIGIN_SAMPLE=\"(.*)\"/) {
        	$VPSID[$matrix_line][124] = $1;
	    }
            if ($line =~ /^NAMESERVER=\"(.*)\"/) {
                $nameservers = $1;
                $nameservers =~ s/ /&/g;
                $VPSID[$matrix_line][125] = "&" . $nameservers . "&";
            }
	    if ($line =~ /^QUOTATIME=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][126] = $1;
    	    }
	    $quota = '';
	    if ($line =~ /^DISK_QUOTA=\"(.*)\"/) {
		$quota = $1;
		if (($quota eq 'yes') || ($quota eq 'Yes')) {
    	    	    $VPSID[$matrix_line][127] = 'Yes';
		}
		else {
    	    	    $VPSID[$matrix_line][127] = 'No';
		}
    	    }
	    if (($line =~ /^DISK_QUOTA=(.*)/) && ($quota eq '')){
		$quota = $1;
		if (($quota eq 'yes') || ($quota eq 'Yes')) {
    	    	    $VPSID[$matrix_line][127] = 'Yes';
		}
		else {
    	    	    $VPSID[$matrix_line][127] = 'No';
		}
    	    }

	    if ($line =~ /^QUOTAUGIDLIMIT=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][128] = $1;
    	    }
	    if ($line =~ /^AVNUMPROC=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][129] = $1;
    	    	$VPSID[$matrix_line][130] = $2;
    	    }
	    elsif ($line =~ /^AVNUMPROC=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][129] = $1;
    	    	$VPSID[$matrix_line][130] = $1;
    	    }
	    # SWAPPAGES (new):
	    if ($line =~ /^SWAPPAGES=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][116] = $1;
    	    	$VPSID[$matrix_line][117] = $2;
    	    }
	    elsif ($line =~ /^SWAPPAGES=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][116] = $1;
    	    	$VPSID[$matrix_line][117] = $1;
    	    }

	    # MEMINFO (new):
	    if ($line =~ /^MEMINFO=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][134] = $1;
    	    }

	    # ip_list:
	    if ($line =~ /^IP_ADDRESS=\"(.*)\"/) {
                $ip_list = $1;
                $ip_list =~ s/ /&/g;
		$combined_ip_list = "&" . $ip_list . "&";
		@raw_ip_list =  split('&', $combined_ip_list);
		shift(@raw_ip_list);
		shift(@raw_ip_list);
		$extra_ip_list = join("&", @raw_ip_list);
		if ($extra_ip_list) {
	                $VPSID[$matrix_line][131] = "&" . $extra_ip_list . "&";
		}
	    }
	    # While this is ugly, we also need to get barriers and limits from the config:

	    # 7 = kmemsize.b
	    # 8 = kmemsize.l
	    if ($line =~ /^KMEMSIZE=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][7] = $1;
    	    	$VPSID[$matrix_line][8] = $2;
    	    }

	    # 12 = lockedpages.b
	    # 13 = lockedpages.l
	    if ($line =~ /^LOCKEDPAGES=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][12] = $1;
    	    	$VPSID[$matrix_line][13] = $2;
    	    }
	    elsif ($line =~ /^LOCKEDPAGES=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][12] = $1;
    	    	$VPSID[$matrix_line][13] = $1;
    	    }
	    
	    # 17 = privvmpages.b
	    # 18 = privvmpages.l
	    if ($line =~ /^PRIVVMPAGES=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][17] = $1;
    	    	$VPSID[$matrix_line][18] = $2;
    	    }
	    elsif ($line =~ /^PRIVVMPAGES=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][17] = $1;
    	    	$VPSID[$matrix_line][18] = $1;
    	    }

	    # 22 = shmpages.b
	    # 23 = shmpages.l
	    if ($line =~ /^SHMPAGES=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][22] = $1;
    	    	$VPSID[$matrix_line][23] = $2;
    	    }
	    elsif ($line =~ /^SHMPAGES=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][22] = $1;
    	    	$VPSID[$matrix_line][23] = $1;
    	    }

	    # 27 = numproc.b
	    # 28 = numproc.l
	    if ($line =~ /^NUMPROC=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][27] = $1;
    	    	$VPSID[$matrix_line][28] = $2;
    	    }
	    elsif ($line =~ /^NUMPROC=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][27] = $1;
    	    	$VPSID[$matrix_line][28] = $1;
    	    }

	    # 32 = physpages.b
	    # 33 = physpages.l
	    if ($line =~ /^PHYSPAGES=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][32] = $1;
    	    	$VPSID[$matrix_line][33] = $2;
    	    }
	    elsif ($line =~ /^PHYSPAGES=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][32] = $1;
    	    	$VPSID[$matrix_line][33] = $1;
    	    }

	    # 37 = vmguarpages.b
	    # 38 = vmguarpages.l
	    if ($line =~ /^VMGUARPAGES=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][37] = $1;
    	    	$VPSID[$matrix_line][38] = $2;
    	    }
	    elsif ($line =~ /^VMGUARPAGES=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][37] = $1;
    	    	$VPSID[$matrix_line][38] = $1;
    	    }

	    # 42 = oomguarpages.b
	    # 43 = oomguarpages.l
	    if ($line =~ /^OOMGUARPAGES=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][42] = $1;
    	    	$VPSID[$matrix_line][43] = $2;
    	    }
	    elsif ($line =~ /^OOMGUARPAGES=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][42] = $1;
    	    	$VPSID[$matrix_line][43] = $1;
    	    }

	    # 47 = numtcpsock.b
	    # 48 = numtcpsock.l
	    if ($line =~ /^NUMTCPSOCK=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][47] = $1;
    	    	$VPSID[$matrix_line][48] = $2;
    	    }
	    elsif ($line =~ /^NUMTCPSOCK=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][47] = $1;
    	    	$VPSID[$matrix_line][48] = $1;
    	    }

	    # 52 = numflock.b
	    # 53 = numflock.l
	    if ($line =~ /^NUMTCPSOCK=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][52] = $1;
    	    	$VPSID[$matrix_line][53] = $2;
    	    }
	    elsif ($line =~ /^NUMTCPSOCK=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][52] = $1;
    	    	$VPSID[$matrix_line][53] = $1;
    	    }

	    # 57 = numpty.b
	    # 58 = numpty.l
	    if ($line =~ /^NUMPTY=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][57] = $1;
    	    	$VPSID[$matrix_line][58] = $2;
    	    }
	    elsif ($line =~ /^NUMPTY=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][57] = $1;
    	    	$VPSID[$matrix_line][58] = $1;
    	    }

	    # 62 = numsiginfo.b
	    # 63 = numsiginfo.l
	    if ($line =~ /^NUMSIGINFO=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][62] = $1;
    	    	$VPSID[$matrix_line][63] = $2;
    	    }
	    elsif ($line =~ /^NUMSIGINFO=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][62] = $1;
    	    	$VPSID[$matrix_line][63] = $1;
    	    }

	    # 67 = tcpsndbuf.b
	    # 68 = tcpsndbuf.l
	    if ($line =~ /^TCPSNDBUF=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][67] = $1;
    	    	$VPSID[$matrix_line][68] = $2;
    	    }
	    elsif ($line =~ /^TCPSNDBUF=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][67] = $1;
    	    	$VPSID[$matrix_line][68] = $1;
    	    }

	    # 72 = tcprcvbuf.b
	    # 73 = tcprcvbuf.l
	    if ($line =~ /^TCPRCVBUF=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][72] = $1;
    	    	$VPSID[$matrix_line][73] = $2;
    	    }
	    elsif ($line =~ /^TCPRCVBUF=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][72] = $1;
    	    	$VPSID[$matrix_line][73] = $1;
    	    }

	    # 77 = othersockbuf.b
	    # 78 = othersockbuf.l
	    if ($line =~ /^OTHERSOCKBUF=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][77] = $1;
    	    	$VPSID[$matrix_line][78] = $2;
    	    }
	    elsif ($line =~ /^OTHERSOCKBUF=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][77] = $1;
    	    	$VPSID[$matrix_line][78] = $1;
    	    }

	    # 82 = dgramrcvbuf.b
	    # 83 = dgramrcvbuf.l
	    if ($line =~ /^DGRAMRCVBUF=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][82] = $1;
    	    	$VPSID[$matrix_line][83] = $2;
    	    }
	    elsif ($line =~ /^DGRAMRCVBUF=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][82] = $1;
    	    	$VPSID[$matrix_line][83] = $1;
    	    }

	    # 87 = numothersock.b
	    # 88 = numothersock.l
	    if ($line =~ /^NUMOTHERSOCK=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][87] = $1;
    	    	$VPSID[$matrix_line][88] = $2;
    	    }
	    elsif ($line =~ /^NUMOTHERSOCK=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][87] = $1;
    	    	$VPSID[$matrix_line][88] = $1;
    	    }

	    # 92 = dcachesize.b
	    # 93 = dcachesize.l
	    if ($line =~ /^DCACHESIZE=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][92] = $1;
    	    	$VPSID[$matrix_line][93] = $2;
    	    }
	    elsif ($line =~ /^DCACHESIZE=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][92] = $1;
    	    	$VPSID[$matrix_line][93] = $1;
    	    }

	    # 97 = numfile.b
	    # 98 = numfile.l
	    if ($line =~ /^NUMFILE=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][97] = $1;
    	    	$VPSID[$matrix_line][98] = $2;
    	    }
	    elsif ($line =~ /^NUMFILE=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][97] = $1;
    	    	$VPSID[$matrix_line][98] = $1;
    	    }

	    # 102 = numiptent.b
	    # 103 = numiptent.l
	    if ($line =~ /^NUMIPTENT=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][102] = $1;
    	    	$VPSID[$matrix_line][103] = $2;
    	    }
	    elsif ($line =~ /^NUMIPTENT=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][102] = $1;
    	    	$VPSID[$matrix_line][103] = $1;
    	    }
	}
	close(CFILE);

	# NAMESERVER must be of format 'ipAddr', but on new VPS's it may not be set - yet. If it is
	# NOT set, we set it to '127.0.0.1' here:
	if (!$VPSID[$matrix_line][125]) {
    	    $VPSID[$matrix_line][125] = "127.0.0.1";
	}
	# QUOTATIME may not be set. If it's not set, we set it to zero:
        if (!$VPSID[$matrix_line][126]) {
	    $VPSID[$matrix_line][126] = "0";
        }
	# DISK_QUOTA may not be set. Valid entries are "yes" or "no". If it's neither, we default to "no":
        if (($VPSID[$matrix_line][127] ne "Yes") && ($VPSID[$matrix_line][127] ne "No")) {
            $VPSID[$matrix_line][127] = "No";
        }
	# QUOTAUGIDLIMIT may not be set. This is for second level quota (user and group disk quota inside the VPS)
        if (!$VPSID[$matrix_line][128]) {
	    $VPSID[$matrix_line][128] = "10000";
        }
    }
}

# Subroutine that feeds the data into CCE:
sub feedthemonster {
	&validate;
	# This is where we actually populate CCE with the gathered info:

	# Check if the VPS is already present in CCE under the old VPS ID:
        @oids = $cce->find('Vserver', {'vpsid' => $currentvpsid});

	# Check if the VPS is already present under a different VPS ID:
	@moved_oids = $cce->find('Vserver', {'ip' => $VPSID[$matrix_line][1], 'hostname' => $VPSID[$matrix_line][125]});

	# Legend: 
	# $#oids : $#moved_oids : action
	#
	#  0 : 0  : update
	# -1 : 0  : update
	# -1 : -1 : create:

	if ($DEBUG == "1") {
	    print "DEB| VPSID: " . $VPSID[$matrix_line][0] . " - oid: " . $#oids . " - moved_oids: " . $#moved_oids . "\n";
	}

        if (($#oids == -1) && ($#moved_oids == -1)) {

                # Object not yet in CCE. Creating new one:
                ($ok) = $cce->create('Vserver', {
                'vpsid' => $VPSID[$matrix_line][0],
                'ip' => $VPSID[$matrix_line][1],
                'hostname' => $VPSID[$matrix_line][2],
                'OSTEMPLATE' => $VPSID[$matrix_line][123],
                'ORIGIN_SAMPLE' => $VPSID[$matrix_line][124],
                'NAMESERVER' => $VPSID[$matrix_line][125],
                'ONBOOT' => $VPSID[$matrix_line][120],
                'is_dir_present' => $VPSID[$matrix_line][132],
                'is_vps_present' => '1',
                'timestamp' => time()
                });
		$new_vps_found = '1'; 			# We found a new VPS, one that we have never seen before and which
							# is not yet in CCE. We do not want to risk to populate CCE with ALL
							# the found new data in one go. So let us just set the most basic info
							# and the timestamp. 
							#
							# We also set $new_vps_found = '1' to run feedthemonster() again.
        }
        else {
	        #print "Object already present in CCE. Updating it.\n";
                # find the system object
		$new_vps_found = '0';
		&validate;

		if (($#oids < 0) && ($#moved_oids == -1)) {
                	($sys_oid) = $cce->find('Vserver', {'vpsid' => $currentvpsid});
                	($ok, $sys) = $cce->get($sys_oid);
		}
		else {
                	($sys_oid) = $cce->find('Vserver', {'ip' => $VPSID[$matrix_line][1], 'hostname' => $VPSID[$matrix_line][2]});
                	($ok, $sys) = $cce->get($sys_oid);
		}

                # If $sys_oid is empty at this point, then we look again and fetch the object of $currentvpsid:
                if ($sys_oid == "") {
                        ($sys_oid) = $cce->find('Vserver', {'vpsid' => $currentvpsid});
                        ($ok, $sys) = $cce->get($sys_oid);
                }
		
		# For debugging and logging purpose we do not write all 122 entries into CCE in one go. Instead we break it down
		# into multiple blocks so that we can troubleshoot /var/log/messages easily for data-blocks with values that CCE
		# did not like. 
		
                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'vpsid' => $VPSID[$matrix_line][0],
                'ip' => $VPSID[$matrix_line][1],
                'ip_list' => $VPSID[$matrix_line][131],
                'hostname' => $VPSID[$matrix_line][2],
                'status' => $VPSID[$matrix_line][3],
                'volume' => '/vz',
                'basedir' => 'private',
                'is_dir_present' => $VPSID[$matrix_line][132],
                'is_vps_present' => '1',
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'avnumproc_b' => $VPSID[$matrix_line][129],
                'avnumproc_l' => $VPSID[$matrix_line][130],

                'swappages' => $VPSID[$matrix_line][114],
                'swappages_m' => $VPSID[$matrix_line][115],
                'swappages_b' => $VPSID[$matrix_line][116],
                'swappages_l' => $VPSID[$matrix_line][117],
                'swappages_f' => $VPSID[$matrix_line][118],

                'meminfo' => $VPSID[$matrix_line][134],

                'kmemsize' => $VPSID[$matrix_line][5],
                'kmemsize_m' => $VPSID[$matrix_line][6],
                'kmemsize_b' => $VPSID[$matrix_line][7],
                'kmemsize_l' => $VPSID[$matrix_line][8],
                'kmemsize_f' => $VPSID[$matrix_line][9],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'lockedpages' => $VPSID[$matrix_line][10],
                'lockedpages_m' => $VPSID[$matrix_line][11],
                'lockedpages_b' => $VPSID[$matrix_line][12],
                'lockedpages_l' => $VPSID[$matrix_line][13],
                'lockedpages_f' => $VPSID[$matrix_line][14],
                'privvmpages' => $VPSID[$matrix_line][15],
                'privvmpages_m' => $VPSID[$matrix_line][16],
                'privvmpages_b' => $VPSID[$matrix_line][17],
                'privvmpages_l' => $VPSID[$matrix_line][18],
                'privvmpages_f' => $VPSID[$matrix_line][19],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'shmpages' => $VPSID[$matrix_line][20],
                'shmpages_m' => $VPSID[$matrix_line][21],
                'shmpages_b' => $VPSID[$matrix_line][22],
                'shmpages_l' => $VPSID[$matrix_line][23],
                'shmpages_f' => $VPSID[$matrix_line][24],
                'numproc' => $VPSID[$matrix_line][25],
                'numproc_m' => $VPSID[$matrix_line][26],
                'numproc_b' => $VPSID[$matrix_line][27],
                'numproc_l' => $VPSID[$matrix_line][28],
                'numproc_f' => $VPSID[$matrix_line][29],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'physpages' => $VPSID[$matrix_line][30],
                'physpages_m' => $VPSID[$matrix_line][31],
                'physpages_b' => $VPSID[$matrix_line][32],
                'physpages_l' => $VPSID[$matrix_line][33],
                'physpages_f' => $VPSID[$matrix_line][34],
                'vmguarpages' => $VPSID[$matrix_line][35],
                'vmguarpages_m' => $VPSID[$matrix_line][36],
                'vmguarpages_b' => $VPSID[$matrix_line][37],
                'vmguarpages_l' => $VPSID[$matrix_line][38],
                'vmguarpages_f' => $VPSID[$matrix_line][39],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'oomguarpages' => $VPSID[$matrix_line][40],
                'oomguarpages_m' => $VPSID[$matrix_line][41],
                'oomguarpages_b' => $VPSID[$matrix_line][42],
                'oomguarpages_l' => $VPSID[$matrix_line][43],
                'oomguarpages_f' => $VPSID[$matrix_line][44],
                'numtcpsock' => $VPSID[$matrix_line][45],
                'numtcpsock_m' => $VPSID[$matrix_line][46],
                'numtcpsock_b' => $VPSID[$matrix_line][47],
                'numtcpsock_l' => $VPSID[$matrix_line][48],
                'numtcpsock_f' => $VPSID[$matrix_line][49],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'numflock' => $VPSID[$matrix_line][50],
                'numflock_m' => $VPSID[$matrix_line][51],
                'numflock_b' => $VPSID[$matrix_line][52],
                'numflock_l' => $VPSID[$matrix_line][53],
                'numflock_f' => $VPSID[$matrix_line][54],
                'numpty' => $VPSID[$matrix_line][55],
                'numpty_m' => $VPSID[$matrix_line][56],
                'numpty_b' => $VPSID[$matrix_line][57],
                'numpty_l' => $VPSID[$matrix_line][58],
                'numpty_f' => $VPSID[$matrix_line][59],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'numsiginfo' => $VPSID[$matrix_line][60],
                'numsiginfo_m' => $VPSID[$matrix_line][61],
                'numsiginfo_b' => $VPSID[$matrix_line][62],
                'numsiginfo_l' => $VPSID[$matrix_line][63],
                'numsiginfo_f' => $VPSID[$matrix_line][64],
                'tcpsndbuf' => $VPSID[$matrix_line][65],
                'tcpsndbuf_m' => $VPSID[$matrix_line][66],
                'tcpsndbuf_b' => $VPSID[$matrix_line][67],
                'tcpsndbuf_l' => $VPSID[$matrix_line][68],
                'tcpsndbuf_f' => $VPSID[$matrix_line][69],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'tcprcvbuf' => $VPSID[$matrix_line][70],
                'tcprcvbuf_m' => $VPSID[$matrix_line][71],
                'tcprcvbuf_b' => $VPSID[$matrix_line][72],
                'tcprcvbuf_l' => $VPSID[$matrix_line][73],
                'tcprcvbuf_f' => $VPSID[$matrix_line][74],
                'othersockbuf' => $VPSID[$matrix_line][75],
                'othersockbuf_m' => $VPSID[$matrix_line][76],
                'othersockbuf_b' => $VPSID[$matrix_line][77],
                'othersockbuf_l' => $VPSID[$matrix_line][78],
                'othersockbuf_f' => $VPSID[$matrix_line][79],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'dgramrcvbuf' => $VPSID[$matrix_line][80],
                'dgramrcvbuf_m' => $VPSID[$matrix_line][81],
                'dgramrcvbuf_b' => $VPSID[$matrix_line][82],
                'dgramrcvbuf_l' => $VPSID[$matrix_line][83],
                'dgramrcvbuf_f' => $VPSID[$matrix_line][84],
                'numothersock' => $VPSID[$matrix_line][85],
                'numothersock_m' => $VPSID[$matrix_line][86],
                'numothersock_b' => $VPSID[$matrix_line][87],
                'numothersock_l' => $VPSID[$matrix_line][88],
                'numothersock_f' => $VPSID[$matrix_line][89],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'dcachesize' => $VPSID[$matrix_line][90],
                'dcachesize_m' => $VPSID[$matrix_line][91],
                'dcachesize_b' => $VPSID[$matrix_line][92],
                'dcachesize_l' => $VPSID[$matrix_line][93],
                'dcachesize_f' => $VPSID[$matrix_line][94],
                'numfile' => $VPSID[$matrix_line][95],
                'numfile_m' => $VPSID[$matrix_line][96],
                'numfile_b' => $VPSID[$matrix_line][97],
                'numfile_l' => $VPSID[$matrix_line][98],
                'numfile_f' => $VPSID[$matrix_line][99],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'numiptent' => $VPSID[$matrix_line][100],
                'numiptent_m' => $VPSID[$matrix_line][101],
                'numiptent_b' => $VPSID[$matrix_line][102],
                'numiptent_l' => $VPSID[$matrix_line][103],
                'numiptent_f' => $VPSID[$matrix_line][104],
                'diskspace' => $VPSID[$matrix_line][105],
                'diskspace_s' => $VPSID[$matrix_line][106],
                'diskspace_h' => $VPSID[$matrix_line][107],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'diskinodes' => $VPSID[$matrix_line][108],
                'diskinodes_s' => $VPSID[$matrix_line][109],
                'diskinodes_h' => $VPSID[$matrix_line][110],
                'laverage' => $VPSID[$matrix_line][111],
                'cpulimit' => $VPSID[$matrix_line][112],
                'cpuunits' => $VPSID[$matrix_line][113],
                'timestamp' => time()
                });

		if (!$VPSID[$matrix_line][120]) { $VPSID[$matrix_line][120] = "Yes" ; }
		if (!$VPSID[$matrix_line][121]) { $VPSID[$matrix_line][121] = '/vz/root/$VEID' ; }
		if (!$VPSID[$matrix_line][122]) { $VPSID[$matrix_line][122] = '/vz/private/$VEID' ; }

                ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'ONBOOT' => $VPSID[$matrix_line][120],
                'VE_ROOT' => $VPSID[$matrix_line][121],
                'VE_PRIVATE' => $VPSID[$matrix_line][122],
                'timestamp' => time()
                });

#                'failcnt' => '0',
#                'failcnt_lastevent' => time(),
#                'failcnt_general' => '0',

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'OSTEMPLATE' => $VPSID[$matrix_line][123],
                'ORIGIN_SAMPLE' => $VPSID[$matrix_line][124],
                'NAMESERVER' => $VPSID[$matrix_line][125],
                'QUOTATIME' => $VPSID[$matrix_line][126],
                'DISK_QUOTA' => $VPSID[$matrix_line][127],
                'QUOTAUGIDLIMIT' => $VPSID[$matrix_line][128],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''

                'volume' => '/vz',
                'basedir' => 'private',
                'is_dir_present' => $VPSID[$matrix_line][132],
                'is_vps_present' => '1',
                #'suspend' => '0',  			# Not yet implemented.
                #'name' => 'admin', 			# Not written by the constructor. GUI handles this value.
                'timestamp' => time()
                });
	}

}


# Validate the settings before we pass them on to CCE:
sub validate {

    # ip:
    if (($VPSID[$matrix_line][1] == 0) && ($VPSID[$matrix_line][2] eq "running") ||
	($VPSID[$matrix_line][1] == 0) && ($VPSID[$matrix_line][2] eq "stopped") ||
	($VPSID[$matrix_line][1] == 0) && ($VPSID[$matrix_line][2] eq "mounted")) {

	# If we arrive at this point, then the entire vzlist maxtrix is shifted left from the 
	# 2nd column onwards. This may happen on new VPS which don't have an IP or hostname set yet. 
	# Means that all the data that we have gathered is junk. How to deal with this?
	# We have the vpsid, we have the tools (vzctl), so we set it manually and start over:
	$bad_info_detected++;
	system("/usr/sbin/vzctl set $currentvpsid --ipadd 127.0.0.1 --save");
    }

    if ($VPSID[$matrix_line][1] == 0) {
	$VPSID[$matrix_line][1] = "127.0.0.1";
    }
    # hostname:
    if ((!$VPSID[$matrix_line][2]) || ($VPSID[$matrix_line][2] eq "-")) {
	$VPSID[$matrix_line][2] = "localhost.localdomain";
    }
    # ostemplate:
    if (!$VPSID[$matrix_line][123]) {
	$VPSID[$matrix_line][123] = 'unknown';
    }

    ## If a VPS is stopped, then a LOT of values gathered from a VPS will be blanks, represented by a '-'. Which is an invalid
    ## character as far as our CCE-Schema is concerned. So we need to replace that with a '0'. But we cannot do that globally,
    ## as a VPS domain or hostname may also have a '-' in it. So we do that for each value independently:

    if ($VPSID[$matrix_line][3] ne "running") {

        $VPSID[$matrix_line][5] = "0";
        $VPSID[$matrix_line][6] = "0";
        $VPSID[$matrix_line][9] = "0";
        $VPSID[$matrix_line][10] = "0";
        $VPSID[$matrix_line][11] = "0";
        $VPSID[$matrix_line][14] = "0";
        $VPSID[$matrix_line][15] = "0";
        $VPSID[$matrix_line][16] = "0";
        $VPSID[$matrix_line][19] = "0";
        $VPSID[$matrix_line][20] = "0";
        $VPSID[$matrix_line][21] = "0";
        $VPSID[$matrix_line][24] = "0";
        $VPSID[$matrix_line][25] = "0";
        $VPSID[$matrix_line][26] = "0";
        $VPSID[$matrix_line][29] = "0";
        $VPSID[$matrix_line][30] = "0";
        $VPSID[$matrix_line][31] = "0";
        $VPSID[$matrix_line][34] = "0";
        $VPSID[$matrix_line][35] = "0";
        $VPSID[$matrix_line][36] = "0";
        $VPSID[$matrix_line][39] = "0";
        $VPSID[$matrix_line][40] = "0";
        $VPSID[$matrix_line][41] = "0";
        $VPSID[$matrix_line][44] = "0";
        $VPSID[$matrix_line][45] = "0";
        $VPSID[$matrix_line][46] = "0";
        $VPSID[$matrix_line][49] = "0";
        $VPSID[$matrix_line][50] = "0";
        $VPSID[$matrix_line][51] = "0";
        $VPSID[$matrix_line][54] = "0";
        $VPSID[$matrix_line][55] = "0";
        $VPSID[$matrix_line][56] = "0";
        $VPSID[$matrix_line][59] = "0";
        $VPSID[$matrix_line][60] = "0";
        $VPSID[$matrix_line][61] = "0";
        $VPSID[$matrix_line][64] = "0";
        $VPSID[$matrix_line][65] = "0";
        $VPSID[$matrix_line][66] = "0";
        $VPSID[$matrix_line][69] = "0";
        $VPSID[$matrix_line][70] = "0";
        $VPSID[$matrix_line][71] = "0";
        $VPSID[$matrix_line][74] = "0";
        $VPSID[$matrix_line][75] = "0";
        $VPSID[$matrix_line][76] = "0";
        $VPSID[$matrix_line][79] = "0";
        $VPSID[$matrix_line][80] = "0";
        $VPSID[$matrix_line][81] = "0";
        $VPSID[$matrix_line][84] = "0";
        $VPSID[$matrix_line][85] = "0";
        $VPSID[$matrix_line][86] = "0";
        $VPSID[$matrix_line][89] = "0";
        $VPSID[$matrix_line][90] = "0";
        $VPSID[$matrix_line][91] = "0";
        $VPSID[$matrix_line][94] = "0";
        $VPSID[$matrix_line][95] = "0";
        $VPSID[$matrix_line][96] = "0";
        $VPSID[$matrix_line][99] = "0";
        $VPSID[$matrix_line][100] = "0";
        $VPSID[$matrix_line][101] = "0";
        $VPSID[$matrix_line][104] = "0";
        $VPSID[$matrix_line][105] = "0";
        $VPSID[$matrix_line][108] = "0";
        $VPSID[$matrix_line][111] = "0";
        $VPSID[$matrix_line][114] = "0";
        $VPSID[$matrix_line][115] = "0";
        $VPSID[$matrix_line][118] = "0";
    }

    # Set 'is_dir_present' if it is undefined:
    if (($VPSID[$matrix_line][132] == "") || (!$VPSID[$matrix_line][132])) {
        $VPSID[$matrix_line][132] = "0";
    }

    # If 'meminfo' is undefined, set it to the default:
    if (($VPSID[$matrix_line][134] == "") || (!$VPSID[$matrix_line][134])) {
        $VPSID[$matrix_line][134] = "privvmpages:1";
    }

}

# Update or populate CCE with the CPU_POWER and CURRENT_CPU (usage) of the master node:
sub cce_update_cpu {

	# This is where we actually populate CCE with the gathered info:
        @oids = $cce->find('VserverMasterNode');
        unless (($CURRENT_CPU == "") || ($NODE_CPU == "")) {
    	    if ($#oids < 0) {
                #print "Object not yet in CCE. Creating new one.\n";
                ($ok) = $cce->create('VserverMasterNode', {
                'CURRENT_CPU' => $CURRENT_CPU,
                'NODE_CPU' => $NODE_CPU
                });
    	    }
    	    else {
                #print "Object already present in CCE. Updating it.\n";
                # find the system object
                ($sys_oid) = $cce->find('VserverMasterNode');
                ($ok, $sys) = $cce->get($sys_oid);
                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'CURRENT_CPU' => $CURRENT_CPU,
                'NODE_CPU' => $NODE_CPU
                });
	    }
	}
}


sub ghostbusters {

    # Compare @newVservers and @oldVservers to find out for which Vserver we have have CODB objects
    # despite them no longer being present according to vzlist. The offending Vserver are added to
    # array @diff:
    
    my (@union, @intersec, @diff, %count);
    $count{$_}++ for (@newVservers, @oldVservers);
    for (keys %count) {
	push @union, $_;
	push @{ $count{$_} > 1 ? \@intersec : \@diff }, $_;
    }

    # Sort the VPSID's numerically in our arrays:
    @diff_unsort = @diff;
    @diff = sort {$a <=> $b} @diff_unsort;
    @intersec_unsort = @intersec;
    @intersec = sort {$a <=> $b} @intersec_unsort;

    # Walk through @intersec and find all VPSID's and confirm they have a file area:
    for $entry (@intersec) {
        # Store them in @intersec_check:
        @intersec_check = $cce->find('Vserver', { 'vpsid' => $entry });
        ($sys_oid) = $cce->find('Vserver', {'vpsid' => $entry});
        ($ok, $is_vpsid) = $cce->get($sys_oid);

	if ($DEBUG == "1") {
	    print "IS-VPSID: " . $is_vpsid->{vpsid} . "\n";
	}

	# Build directory path:
	$isect_vps_dir = "/vz/private/" . $is_vpsid->{vpsid};
	if (! -d $isect_vps_dir) {

	    if ($DEBUG == "1") {
		print "VPSID " . $is_vpsid->{vpsid} . " has no file area!\n";
	    }

    	    # To prevent that VPS's are actually destroyed on the file system level we change the VPSID
    	    # of the VPS in the Vserver object to 9999 and also set is_vps_present and is_dir_present to 0:
    	    ($ok) = $cce->set($sys_oid, '',{
                'is_vps_present' => '0',
                'is_dir_present' => '0',
                'vpsid' => '9999',
                'timestamp' => time()
                });

    	    # As a final step we can now safely delete the offending Vserver object:
    	    if ($sys_oid ne "") {
    		if ($DEBUG == "1") {
    		    print "Destroying OID $sys_oid from VPSID " . $is_vpsid->{vpsid} . "\n";
    		}
                ($ok) = $cce->destroy($sys_oid);
    	    }
	}
    }

    # Walk through the offending Vserver objects from array @diff:
    if ($DEBUG == "1") {
	print "diff: @diff \n";
        print "intersec: @intersec \n";
    }

# This blows up right in my face if uncommented:
#
#    
#    for $entry (@diff) {
#	# Store them in @double:
#	@double = $cce->find('Vserver', { 'vpsid' => $entry });
#	# To prevent that VPS's are actually destroyed on the file system level we change the VPSID
#	# of the VPS in the Vserver object to 9999 and also set is_vps_present and is_dir_present to 0:
#        ($ok) = $cce->set($double[0], '',{
#                'is_vps_present' => '0',
#                'is_dir_present' => '0',
#		'vpsid' => '9998',
#                'timestamp' => time()
#                });
#        # As a final step we can now safely delete the offending Vserver object:
#	if ($double[0] ne "") {
#	    ($ok) = $cce->destroy($double[0]);
#	}
#    }

    # Finally there may be cases where we have Vserver objects left over that are NOT matching against
    # vzlist at all. So we walk through all of them and check if they have a file area:
    @allVservers = $cce->find('Vserver');
    
    # Build an array of all VPSIDs of all Vservers:
    @anyVPSID = ();
    foreach $all_my_entries (@allVservers) {
	($ok, $all_my_vpsid) = $cce->get($all_my_entries);
	push (@anyVPSID, $all_my_vpsid->{vpsid});
    }
    # Sort VPSIDs in @anyVPSID;
    @anyVPSID_unsort = @anyVPSID;
    @anyVPSID = sort {$a <=> $b} @anyVPSID_unsort;
    
    if ($DEBUG == "1") {
	print "allVservers: @allVservers \n";
	print "anyVPSID   : @anyVPSID    \n";
    }
    
    foreach $vps_entry (@allVservers) {
        ($ok, $one_vps_of_all) = $cce->get($vps_entry);
        if ($DEBUG == "1") {
	    print "ALL-VPSID: " . $one_vps_of_all->{vpsid} . "\n";
	}
	# Build directory path:
	$all_vps_dir = "/vz/private/" . $one_vps_of_all->{vpsid};
	if (! -d $all_vps_dir) {
	    if ($DEBUG == "1") {
		print "ALL-VPSID " . $one_vps_of_all->{vpsid} . " has no file area!\n";
	    }

	    # To prevent that VPS's are actually destroyed on the file system level we change the VPSID
	    # of the VPS in the Vserver object to 9999 and also set is_vps_present and is_dir_present to 0:
    	    ($ok) = $cce->set($vps_entry, '',{
                'is_vps_present' => '0',
                'is_dir_present' => '0',
		'vpsid' => '9997',
                'timestamp' => time()
                });
    	    # As a final step we can now safely delete the offending Vserver object:
	    if ($vps_entry ne "") {
		($ok) = $cce->destroy($vps_entry);
	    }
	}	
    }
}

# Handle IP address allocation:

sub ip_alloc {

    # Find existing vpsIPalloc objects:
    @old_alloc = $cce->findx('vpsIPalloc', {'vpsid' => $currentvpsid});
    if ($#old_alloc == -1) {
        # Nothing found in CCE
        if (@raw_ip_list) {

            # But we found extra IPs in the config file
            for $alloc_entry (@old_alloc) {
                # Destroy old allocation entries:
                ($ok) = $cce->destroy($alloc_entry);
            }

            # Write new ones:
            foreach $jummy_ip (@raw_ip_list) {
                if ($jummy_ip ne "") {
                    ($ok) = $cce->create('vpsIPalloc', {
                        'vpsid' => $currentvpsid,
                        'min' => $jummy_ip,
                        'creation_time' => time()
                    });
                }
            }
        }
    }
} 

$cce->bye('SUCCESS');
exit(0);

# Legend of Matrix columns:

# 0 = vpsid
# 1 = ip
# 2 = hostname
# 3 = status
# 4 = numproc
# 5 = kmemsize
# 6 = kmemsize.m
# 7 = kmemsize.b
# 8 = kmemsize.l
# 9 = kmemsize.f
# 10 = lockedpages
# 11 = lockedpages.m
# 12 = lockedpages.b
# 13 = lockedpages.l
# 14 = lockedpages.f
# 15 = privvmpages
# 16 = privvmpages.m
# 17 = privvmpages.b
# 18 = privvmpages.l
# 19 = privvmpages.f
# 20 = shmpages
# 21 = shmpages.m
# 22 = shmpages.b
# 23 = shmpages.l
# 24 = shmpages.f
# 25 = numproc
# 26 = numproc.m
# 27 = numproc.b
# 28 = numproc.l
# 29 = numproc.f
# 30 = physpages
# 31 = physpages.m
# 32 = physpages.b
# 33 = physpages.l
# 34 = physpages.f
# 35 = vmguarpages
# 36 = vmguarpages.m
# 37 = vmguarpages.b
# 38 = vmguarpages.l
# 39 = vmguarpages.f
# 40 = oomguarpages
# 41 = oomguarpages.m
# 42 = oomguarpages.b
# 43 = oomguarpages.l
# 44 = oomguarpages.f
# 45 = numtcpsock
# 46 = numtcpsock.m
# 47 = numtcpsock.b
# 48 = numtcpsock.l
# 49 = numtcpsock.f
# 50 = numflock
# 51 = numflock.m
# 52 = numflock.b
# 53 = numflock.l
# 54 = numflock.f
# 55 = numpty
# 56 = numpty.m
# 57 = numpty.b
# 58 = numpty.l
# 59 = numpty.f
# 60 = numsiginfo
# 61 = numsiginfo.m
# 62 = numsiginfo.b
# 63 = numsiginfo.l
# 64 = numsiginfo.f
# 65 = tcpsndbuf
# 66 = tcpsndbuf.m
# 67 = tcpsndbuf.b
# 68 = tcpsndbuf.l
# 69 = tcpsndbuf.f
# 70 = tcprcvbuf
# 71 = tcprcvbuf.m
# 72 = tcprcvbuf.b
# 73 = tcprcvbuf.l
# 74 = tcprcvbuf.f
# 75 = othersockbuf
# 76 = othersockbuf.m
# 77 = othersockbuf.b
# 78 = othersockbuf.l
# 79 = othersockbuf.f
# 80 = dgramrcvbuf
# 81 = dgramrcvbuf.m
# 82 = dgramrcvbuf.b
# 83 = dgramrcvbuf.l
# 84 = dgramrcvbuf.f
# 85 = numothersock
# 86 = numothersock.m
# 87 = numothersock.b
# 88 = numothersock.l
# 89 = numothersock.f
# 90 = dcachesize
# 91 = dcachesize.m
# 92 = dcachesize.b
# 93 = dcachesize.l
# 94 = dcachesize.f
# 95 = numfile
# 96 = numfile.m
# 97 = numfile.b
# 98 = numfile.l
# 99 = numfile.f
# 100 = numiptent
# 101 = numiptent.m
# 102 = numiptent.b
# 103 = numiptent.l
# 104 = numiptent.f
# 105 = diskspace
# 106 = diskspace.s
# 107 = diskspace.h
# 108 = diskinodes
# 109 = diskinodes.s
# 110 = diskinodes.h
# 111 = laverage
# 112 = cpulimit
# 113 = cpuunits

# 114 = swappages
# 115 = swappages.m
# 116 = swappages.b
# 117 = swappages.l
# 118 = swappages.f

# 119 = hostname (when ultralong)

### From config file:
# 120 = onboot
# 121 = ve_root
# 122 = ve_private
# 123 = ostemplate
# 124 = origin_sample
# 125 = nameserver
# 126 = quotatime
# 127 = disk_quota
# 128 = quotaugidlimit
# 129 = avnumproc_b
# 130 = avnumproc_l
# 131 = ip_list (IP address allocation)

### From file operations:
# 132 = is_dir_present (is VPS private area present?)
# 133 = is_vps_present (if vzlist says VPS is there this is set to 1)

# 134 = meminfo (optional, from config)

