#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: update_ipalloc.pl,v 1.0.0.0 Mon 31 Dec 2007 09:33:59 PM CET mstauber Exp $
# Copyright 2006-2012 Solarspeed Ltd. All rights reserved.
#
# Handler which writes / updates VZ config files.
#

use CCE;
use POSIX qw(isalpha);

# Debugging switch:
$DEBUG = "0";
if ($DEBUG)
{
        use Sys::Syslog qw( :DEFAULT setlogsock);
}

my $cce = new CCE('Domain' => 'base-vserver');

$cce->connectfd();
#$cce->connectuds(); # only for debugging

($sys_oidmasternode) = $cce->find('VserverMasterNode');
($ok, $VserverMasterNode) = $cce->get($sys_oidmasternode);
$vpsid = $VserverMasterNode->{gui_list_ip};
&debug_msg("VPSID: $vpsid \n");

##
# IP address allocation stuff:
##

# Find the relevant objects through their vpsid in vpsIPalloc objects:
@oids = $cce->find('vpsIPalloc', {'vpsid' => $vpsid});
if ($DEBUG == 1) {
	&debug_msg("1st: Found $#oids IPs for this VPS \n");
}
if ($#oids == -1) {
    if ($DEBUG == 1) {
	&debug_msg("Unable to determine VPSID based on vpsIPalloc \n");
    }
    #$cce->bye('FAIL', "[[base-vserver.unableToDetermineVPSID]]");
    #exit(1);
    $cce->bye('SUCCESS');
    exit(0);
}
else {
    # VPSID found in CCE.
    foreach $line (@oids) {
            ($ok, $system) = $cce->get($line);
            $func = push(@ips, $system->{min});
    }
    # How many IP's we got?
    $num = $#ips;
    if ($DEBUG == 1) {
	&debug_msg("2nd: Found $num IPs for this VPS \n");
    }
    # Sort the IPs:
    @sorted_ips = sort { $a cmp $b } @ips;
    # Line them up in the single string $iplist, separated by spaces:
    foreach $value (@sorted_ips) {
        if ($num > 0) {
            $iplist .= $value . ";";
            $num--;
        }
        else {
            $iplist .= $value;
        }
    }
}


# Find the relevant object through its vpsid in Vserver:
@oids = $cce->find('Vserver', {'vpsid' => $vpsid});
if ($#oids < 0) {
    $cce->bye('FAIL', "[[base-vserver.unableToDetermineVPSID]]");
    if ($DEBUG == 1) {
	&debug_msg("VPSID not found in CCE. \n");
    }
    exit(1);
}
else {
    # get the system object
    ($sys_oid) = $cce->find('Vserver', {'vpsid' => $vpsid});
    ($ok, $vpsdata) = $cce->get($sys_oid);

    if ($ok) {
        if ($DEBUG == 1) {
	    &debug_msg("VPSDATA not found in CCE. \n");
        }
        vps_write_conf(
            $vpsdata->{'vpsid'},
            $vpsdata->{'ip'},
            $iplist
        );
    }
    else {
        if ($DEBUG == 1) {
	    &debug_msg("VPSDATA retrieval errored out. \n");
        }
    }
}

sub vps_write_conf {

        my $vpsid                       = shift;
        my $ip                          = shift;
        my $iplist                      = shift;

        # Apply changes to VPS (regardless if it is running or stopped) and rewrite config:
        system("/usr/sbin/vzctl set $vpsid --ipdel all --ipadd $ip --save");

	# Add allocated extra IP addresses in one go:	
	$new_ip_list = $iplist;
	$new_ip_list =~ s/;/ /g;
	if ($DEBUG == 1) {
	    	&debug_msg("iplist: $iplist \n");
	    	&debug_msg("new_ip_list: $new_ip_list \n");
	}
	system("/usr/sbin/vzctl set $vpsid --ipadd \"$new_ip_list\" --save");

}

# We need to say 'SUCCESS' or CCE will bitch:
$cce->bye('SUCCESS');
exit(0);

sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        $user = $ENV{'USER'};
        setlogsock('unix'); 
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}
