#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: vzbackup.pl, v 2.0.0-1 Sun 30 Oct 2011 04:54:23 AM CET mstauber Exp $
# Copyright 2006-2011 Stauber Multimedia Design. All rights reserved.
#
# Handler which reads VserverMasterNode and removes or writes /etc/cron.d/vps-backup.cron
# to enable or disable the local and remote backup of VPS's.
#
# It has been extended to handle both the old RSYNC backup as well as the new VZdump.

use CCE;
use POSIX qw(isalpha);

my $cce = new CCE;
$cce->connectfd();
#$cce->connectuds();

$oid = $cce->event_oid();
$obj = $cce->event_object();

@oids = $cce->find('VserverMasterNode');
if (!defined($oids[0])) {
	print STDERR "VZBACKUP: Sorry, no VserverMasterNode object in CCE found!\n";
	exit 1;
}

$vserver_cronCrontab = "/etc/cron.d/vps-backup.cron";
$vserver_cronConfig = "/etc/vz/aconf/vps-mn-backup.conf";

if (!-d "/etc/vz/aconf") {
    system("/bin/mkdir /etc/vz/aconf");
}

($ok, $solbackupValues) = $cce->get($oids[0]);

if ($ok) {

    solbackupSetCrontab(
        $solbackupValues->{'MN_LocalBackupEnabled'},
        $solbackupValues->{'MN_RemoteBackupEnabled'},
        $solbackupValues->{'backupSU_local'},
        $solbackupValues->{'backupMO_local'},
        $solbackupValues->{'backupTU_local'},
        $solbackupValues->{'backupWE_local'},
        $solbackupValues->{'backupTH_local'},
        $solbackupValues->{'backupFR_local'},
        $solbackupValues->{'backupSA_local'},
        $solbackupValues->{'backupTime_local'},
        $solbackupValues->{'backupSU_remote'},
        $solbackupValues->{'backupMO_remote'},
        $solbackupValues->{'backupTU_remote'},
        $solbackupValues->{'backupWE_remote'},
        $solbackupValues->{'backupTH_remote'},
        $solbackupValues->{'backupFR_remote'},
        $solbackupValues->{'backupSA_remote'},
        $solbackupValues->{'backupTime_remote'},
	$solbackupValues->{'MN_BackupRemoteTargetHost'},
	$solbackupValues->{'MN_BackupRemoteTargetUser'},
	$solbackupValues->{'MN_BackupRemoteTargetUserPass'},
	$solbackupValues->{'MN_BackupRemoteTargetPort'},
	$solbackupValues->{'MN_BackupRemoteLocation'},
	$solbackupValues->{'MN_BackupLocalLocation'},
	$solbackupValues->{'vzdump_enabled'},
	$solbackupValues->{'vzdump_maxfiles'},
	$solbackupValues->{'vzdump_ionice'},
	$solbackupValues->{'vzdump_bwlimit'},
	$solbackupValues->{'rsync_enabled'}
        );
}

$cce->bye('SUCCESS');
exit(0);

##
## Sort the stuff for the crontab file
##

sub solbackupSetCrontab {

    my $MN_LocalBackupEnabled 	= shift;
    my $MN_RemoteBackupEnabled 	= shift;
    my $backupSU_local 		= shift;
    my $backupMO_local 		= shift;
    my $backupTU_local 		= shift;
    my $backupWE_local 		= shift;
    my $backupTH_local 		= shift;
    my $backupFR_local 		= shift;
    my $backupSA_local 		= shift;
    my $backupTime_local	= shift;
    my $backupSU_remote		= shift;
    my $backupMO_remote		= shift;
    my $backupTU_remote		= shift;
    my $backupWE_remote		= shift;
    my $backupTH_remote		= shift;
    my $backupFR_remote		= shift;
    my $backupSA_remote		= shift;
    my $backupTime_remote	= shift;

    my $MN_BackupRemoteTargetHost   		= shift;
    my $MN_BackupRemoteTargetUser   		= shift;
    my $MN_BackupRemoteTargetUserPass  		= shift;
    my $MN_BackupRemoteTargetPort   		= shift;
    my $MN_BackupRemoteLocation   		= shift;
    my $MN_BackupLocalLocation   		= shift;

    my $vzdump_enabled				= shift;
    my $vzdump_maxfiles				= shift;
    my $vzdump_ionice				= shift;
    my $vzdump_bwlimit				= shift;
    my $rsync_enabled				= shift;

    if (($MN_LocalBackupEnabled == "1") || ($MN_RemoteBackupEnabled == "1")) {
	$backup_enabled = "1";
    }
    else {
	$backup_enabled = "0";
    }

    ($h_local, $m_local) = split(':', $backupTime_local);
    my $crontab_entry_local = "$m_local $h_local * * ";
    my $hv_local = ($h_local+1);
    if ($hv_local >=24) { $hv_local = 0; } 
    my $vserver_cron_time_local = "$m_local $hv_local * *";
    my $hu_local = ($hv_local+'1');

    my $tmp_crontab_entry_local =
        ($backupSU_local eq 1 ? "0," : "")
      . ($backupMO_local eq 1 ? "1," : "")
      . ($backupTU_local eq 1 ? "2," : "")
      . ($backupWE_local eq 1 ? "3," : "")
      . ($backupTH_local eq 1 ? "4," : "")
      . ($backupFR_local eq 1 ? "5," : "")
      . ($backupSA_local eq 1 ? "6," : "");

    $tmp_crontab_entry_local =~ s/,$//;
    $tmp_crontab_entry_local = "*" if length($tmp_crontab_entry_local) < 1;
    $my_local = $crontab_entry_local;
    $my_local .= $tmp_crontab_entry_local;
    $crontab_entry_local .= $tmp_crontab_entry_local . " root /usr/sausalito/sbin/vz_vserver_backup.pl local >/dev/null 2>&1\n";

    ($h_remote, $m_remote) = split(':', $backupTime_remote);
    my $crontab_entry_remote = "$m_remote $h_remote * * ";
    my $hv_remote = ($h_remote+1);
    if ($hv_remote >=24) { $hv_remote = 0; } 
    my $vserver_cron_time_remote = "$m_remote $hv_remote * *";
    my $hu_remote = ($hv_remote+'1');

    my $tmp_crontab_entry_remote =
        ($backupSU_remote eq 1 ? "0," : "")
      . ($backupMO_remote eq 1 ? "1," : "")
      . ($backupTU_remote eq 1 ? "2," : "")
      . ($backupWE_remote eq 1 ? "3," : "")
      . ($backupTH_remote eq 1 ? "4," : "")
      . ($backupFR_remote eq 1 ? "5," : "")
      . ($backupSA_remote eq 1 ? "6," : "");

    $tmp_crontab_entry_remote =~ s/,$//;
    $tmp_crontab_entry_remote = "*" if length($tmp_crontab_entry_remote) < 1;
    $my_remote = $crontab_entry_remote;
    $my_remote .= $tmp_crontab_entry_remote;
    $crontab_entry_remote .= $tmp_crontab_entry_remote . " root /usr/sausalito/sbin/vz_vserver_backup.pl remote >/dev/null 2>&1\n";

    $combined = "0";
    if (($my_local eq $my_remote) && ((($MN_LocalBackupEnabled == "1") && ($MN_RemoteBackupEnabled == "1")))) {
	$crontab_entry_both = $my_local . " root /usr/sausalito/sbin/vz_vserver_backup.pl both >/dev/null 2>&1\n";
	$combined = "1";
    }

    ## VZdump:
    if ($vzdump_enabled == "1") {
	$crontab_entry_both = $my_local . " root /usr/sausalito/sbin/vz_vserver_backup.pl both >/dev/null 2>&1\n";
	$combined = "1";
    }
	
    if ($backup_enabled == 1) {
	open(CRON, ">$vserver_cronCrontab") || die "Can't write to crontab";
	print CRON "# vzbackup - Aventurin{e} Backup \n";
        print CRON "# DO NOT EDIT! THIS FILE IS GENERATED AUTOMATICALLY THROUGH THE GUI!\n";
	if ($combined == "0") {
		if ($MN_LocalBackupEnabled == "1") {
    		    print CRON $crontab_entry_local;
    		}
    		if ($MN_RemoteBackupEnabled == "1") {
    		    print CRON $crontab_entry_remote;
    		}
    	}
    	else {
	    print CRON $crontab_entry_both;
    	}
    	
        print CRON "\n";
        print CRON "\n";
        close CRON;
	system("/bin/chmod 644 $vserver_cronCrontab");
    }
    else {
	# Delete server cron:
	if ( -e $vserver_cronCrontab ) {
	    unlink($vserver_cronCrontab);
	}
    }
    
    # CROND restart:
    system("/etc/init.d/crond stop > /dev/null 2>&1");
    system("/etc/init.d/crond start > /dev/null 2>&1");

    # Write cronConfig:


    open(CRONC, ">$vserver_cronConfig") || die "Can't write to $vserver_cronConfig";
    print CRONC "# vzbackup - Aventurin{e} Backup \n";
    print CRONC "# DO NOT EDIT! THIS FILE IS GENERATED AUTOMATICALLY THROUGH THE GUI!\n";
    print CRONC "\n";
    print CRONC "MN_LocalBackupEnabled $MN_LocalBackupEnabled\n";
    print CRONC "MN_RemoteBackupEnabled $MN_RemoteBackupEnabled\n";
    print CRONC "backupSU $backupSU\n";
    print CRONC "backupMO $backupMO\n";
    print CRONC "backupTU $backupTU\n";
    print CRONC "backupWE $backupWE\n";
    print CRONC "backupTH $backupTH\n";
    print CRONC "backupFR $backupFR\n";
    print CRONC "backupSA $backupSA\n";
    print CRONC "backupTime $backupTime\n";

    print CRONC "backupSU_local $backupSU_local\n";
    print CRONC "backupMO_local $backupMO_local\n";
    print CRONC "backupTU_local $backupTU_local\n";
    print CRONC "backupWE_local $backupWE_local\n";
    print CRONC "backupTH_local $backupTH_local\n";
    print CRONC "backupFR_local $backupFR_local\n";
    print CRONC "backupSA_local $backupSA_local\n";
    print CRONC "backupTime_local $backupTime_local\n";

    print CRONC "backupSU_remote $backupSU_remote\n";
    print CRONC "backupMO_remote $backupMO_remote\n";
    print CRONC "backupTU_remote $backupTU_remote\n";
    print CRONC "backupWE_remote $backupWE_remote\n";
    print CRONC "backupTH_remote $backupTH_remote\n";
    print CRONC "backupFR_remote $backupFR_remote\n";
    print CRONC "backupSA_remote $backupSA_remote\n";
    print CRONC "backupTime_remote $backupTime_remote\n";

    print CRONC "MN_BackupRemoteTargetHost $MN_BackupRemoteTargetHost\n";
    print CRONC "MN_BackupRemoteTargetUser $MN_BackupRemoteTargetUser\n";
    print CRONC "MN_BackupRemoteTargetUserPass $MN_BackupRemoteTargetUserPass\n";
    print CRONC "MN_BackupRemoteTargetPort $MN_BackupRemoteTargetPort\n";
    print CRONC "MN_BackupRemoteTargetIP $MN_BackupRemoteTargetIP\n";
    print CRONC "MN_BackupRemoteLocation $MN_BackupRemoteLocation\n";
    print CRONC "MN_BackupLocalLocation $MN_BackupLocalLocation\n";

    print CRONC "\n";
    print CRONC "vzdump_enabled $vzdump_enabled\n";
    print CRONC "vzdump_maxfiles $vzdump_maxfiles\n";
    print CRONC "vzdump_ionice $vzdump_ionice\n";
    print CRONC "vzdump_bwlimit $vzdump_bwlimit\n";
    print CRONC "rsync_enabled $rsync_enabled\n";
    print CRONC "\n";

    close CRONC;
    system("/bin/chmod 600 $vserver_cronConfig");

}

exit 0;

